use crate::services::service::{Request, Service};
use crate::store::nsobject::NsObject;
use std::{fs::OpenOptions, io::Write};

pub struct Download<'a> {
    request: Request<'a>,
    obj_path: String,
}

impl<'a> Download<'a> {
    pub fn new(service: &'a Service) -> Self {
        Download {
            request: Request::new(service),
            obj_path: String::new(),
        }
    }

    pub fn set_obj_path(mut self, obj_path: String) -> Self {
        self.request.get(obj_path.clone());
        self.obj_path = obj_path
            .strip_prefix("/")
            .unwrap_or(&self.obj_path.clone())
            .to_string();
        self
    }

    pub async fn send(&mut self) -> Result<(), reqwest::Error> {
        let res = self.request.send().await;
        let body = res.unwrap().bytes().await?;
        let mut file = OpenOptions::new()
            .write(true)
            .create(true)
            .open(self.obj_path.clone())
            .unwrap();

        file.write_all(&body.to_vec()).unwrap();
        NsObject::from_local_path(&self.obj_path.clone().into())
            .save()
            .unwrap();
        Ok(())
    }
}
